/*###############################################################################
#                                                                               #
# Project:     Memorywatcher for Topfield 5000 and TMS                          #
#                                                                               #
# Description: Very simple TAP to watch the memory heap of TF5000 or TMS.       #
#              Keys: "Subtitle" ends the program, without menu                  #
#                    "Red" in TMSCommander ends the program                     #
#                                                                               #
# Author:      Bjrn Bckle, Bjoern@Boeckle-net.de                              #
# Adjustments: xyzzyx (TMS changes)                                             #
#                                                                               #
# Comments:    Use it at your own risk                                          #
#                                                                               #
# History:                                                                      #
# Version    When         Who                            What                   #
# ------- ----------- -------------  ------------------------------------------ #
#  v1.0   30.05.2004  Bjrn Bckle   First release                              #
#  v1.1   18.08.2009  xyzzyx         Adjustments for TMS                        #
#  v1.1a  15.08.2010  xyzzyx         support for TAPtoDate                      #
#  v1.1b  21.09.2010  xyzzyx         support for TMSCommander                   #
#  v1.1c  21.09.2010  xyzzyx         bugfix for missing display on newer fw     #
#  v1.1d  18.11.2010  xyzzyx         added FireBirdLib and KeyTranslate         #
#                                                                               #
###############################################################################*/


#include "tap.h"
#ifdef _TMS_ // xyzzyx 100921
#include "TMSCommander.h" // xyzzyx 100921
#endif // xyzzyx 100921

// xyzzyx 101118
#ifdef _TMS_
#include "libFireBird.h"   // tested with FireBirdLib version "2010-11-14"
#endif

// xyzzyx 090818
//#include "tools.h"
#define ID_BASE 0x80001000

#define ID_MEMORY_WATCHER		ID_BASE  +	0x06
#define PGM_NAME		"MemoryWatcher" // xyzzyx 100815
#define PGM_VERS		"V1.1d" // xyzzyx 100815
#define PGM_FULL		PGM_NAME " " PGM_VERS // xyzzyx 100815

TAP_ID( 0x81AFFE06 ); // xyzzyx 100815 changed
TAP_PROGRAM_NAME(PGM_FULL); // xyzzyx 100815 changed
TAP_PROGRAM_VERSION(PGM_VERS); // xyzzyx 100815
TAP_AUTHOR_NAME("xyzzyx");
TAP_DESCRIPTION("Watch the memory usage for TF5000 (Bjrn Bckle) and TMS");
TAP_ETCINFO(__DATE__);

static dword g_rgnMem;
static dword g_rgn;

#ifdef _TMS_ // xyzzyx 100921 >>>
// TMSCommander_handler
static dword TMSCommander_handler(dword param1)
{
	switch (param1)
	{
		case TMSCMDR_Capabilities:
			return (dword)(TMSCMDR_CanBeStopped); //(TMSCMDR_HaveMenu | TMSCMDR_CanBeStopped | TMSCMDR_HaveUserEvent);

	  case TMSCMDR_Menu:
		//call your TAP's menu
			return TMSCMDR_OK;

		case TMSCMDR_Stop:
		//Exit your TAP
#ifdef _TMS_ // xyzzyx 100921
			TAP_Osd_FillBox(g_rgn, 0, 0, 720, 576, COLOR_None); // xyzzyx 100921
			TAP_Osd_Sync(); // xyzzyx 100921
#endif	// xyzzyx 100921
			TAP_Osd_Delete( g_rgn );
			TAP_Osd_Delete( g_rgnMem );
#ifdef _TMS_ // xyzzyx 100921
			TAP_Osd_Sync(); // xyzzyx 100921
#endif	// xyzzyx 100921
			TAP_Exit();
			return TMSCMDR_OK;

		case TMSCMDR_UserEvent:
		//call your TAP's userevent
			return TMSCMDR_OK;

		case TMSCMDR_Exiting:
		//TMSCommander is exiting - do you want to do anything?
			return TMSCMDR_OK;

		default:
		//unknown TMSCommander function
			return TMSCMDR_UnknownFunction;
	}
}
#endif // xyzzyx 100921 <<<

dword TAP_EventHandler( word event, dword param1, dword param2 )
{
	int nWidth, nHeight, xStart, yStart;
// xyzzyx 090818
#ifdef _TMS_
	UNUSED(param2);

  if (event == EVT_TMSCommander) // xyzzyx 100921
		return TMSCommander_handler(param1); // xyzzyx 100921
#endif

	if ( event == EVT_IDLE )
	{
			char strName[150];
			dword total, free, avai;

			TAP_MemInfo(&total, &free, &avai);
			TAP_SPrint(strName, "Total: %d -- Free: %d -- Avai: %d", total, free, avai );

// xyzzyx 090818
#ifdef _TMS_
			nWidth  = 425;																							// fixed width
#else
			nWidth  = TAP_Osd_GetW( strName, 0, FNT_Size_1622 );				// calculate width of message
#endif
			xStart  = (720-nWidth)/2;
			nHeight = 24;
			yStart  = 37 - TAP_GetSystemVar(SYSVAR_IBoxPos)*7;
			if ( yStart < 1 ) yStart = 1;
			TAP_Osd_Draw3dBoxFill( g_rgnMem, xStart-10, yStart -2, nWidth + 20, nHeight + 4,

// xyzzyx 090818
#ifdef _TMS_
					COLOR_White, COLOR_Gray, COLOR_DarkBlue );						// Draw frame c1, c2, fc
#else
					RGB(21,21,21), RGB(19,19,19 ), RGB(3, 5, 10 ) );			// Draw frame
#endif

			TAP_Osd_PutS(g_rgnMem, xStart, yStart, xStart + nWidth, strName,

// xyzzyx 090818
#ifdef _TMS_
					COLOR_White, COLOR_DarkBlue, 0, FNT_Size_1622,
#else
					RGB(28, 28, 28), RGB(3, 5, 10 ), 0, FNT_Size_1622,		// RGB(3,5,10)
#endif

					FALSE, ALIGN_CENTER);									// print message
			TAP_Osd_Copy( g_rgnMem, g_rgn, xStart -10, yStart-2, nWidth +20, nHeight + 4, xStart-10, yStart-2, FALSE );
#ifdef _TMS_ // xyzzyx 100921
			TAP_Osd_Sync(); // xyzzyx 100921
#endif // xyzzyx 100921
	}

	if ( event == EVT_KEY )
	{
		if ( param1 == RKEY_Subt )
		{
#ifdef _TMS_ // xyzzyx 100921
			TAP_Osd_FillBox(g_rgn, 0, 0, 720, 576, COLOR_None); // xyzzyx 100921
			TAP_Osd_Sync(); // xyzzyx 100921
#endif	// xyzzyx 100921
			TAP_Osd_Delete( g_rgn );
			TAP_Osd_Delete( g_rgnMem );
#ifdef _TMS_ // xyzzyx 100921
			TAP_Osd_Sync(); // xyzzyx 100921
#endif	// xyzzyx 100921
			TAP_Exit();
			return 0;
		}
	}
    return param1;
}





int TAP_Main(void)
{
// xyzzyx 101118
#ifdef _TMS_
	KeyTranslate(TRUE, &TAP_EventHandler);
#endif
	g_rgn = TAP_Osd_Create( 0, 0, 720, 576, 0, FALSE );		// create rgn-handle
	g_rgnMem = TAP_Osd_Create( 0, 0, 720, 576, 0, OSD_Flag_MemRgn );	// create memory rgn-handle


	return 1;
}


